import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.time.LocalDate;

class CalendarGenerator {

    // Function to check if a year is a leap year
    public static boolean isLeapYear(int year) {
        return (year % 4 == 0 && (year % 100 != 0 || year % 400 == 0));
    }

    // Function to get the number of days in a month
    public static int getDaysInMonth(int month, int year) {
        switch (month) {
            case 2:
                return isLeapYear(year) ? 29 : 28;
            case 4: case 6: case 9: case 11:
                return 30;
            default:
                return 31;
        }
    }

    // Function to get the day of the week for the first day of the month
    public static int getDayOfWeek(int month, int year) {
        if (month < 3) {
            month += 12;
            year -= 1;
        }
        int k = year % 100;
        int j = year / 100;
        int day = 1 + (13 * (month + 1)) / 5 + k + (k / 4) + (j / 4) + 5 * j;
        return day % 7;
    }

    // Function to print a single month's calendar
    public static String getMonthCalendar(int month, int year) {
        String[] months = {"January", "February", "March", "April", "May", "June",
                "July", "August", "September", "October", "November", "December"};

        int daysInMonth = getDaysInMonth(month, year);
        int dayOfWeek = getDayOfWeek(month, year);

        StringBuilder calendar = new StringBuilder();
        calendar.append("\n-------------------------------------\n");
        calendar.append("           " + months[month - 1] + " " + year + "\n");
        calendar.append("-------------------------------------\n");
        calendar.append(" Sun  Mon  Tue  Wed  Thu  Fri  Sat\n");

        for (int i = 0; i < dayOfWeek; i++) {
            calendar.append("     ");
        }

        LocalDate today = LocalDate.now();
        for (int day = 1; day <= daysInMonth; day++) {
            if (year == today.getYear() && month == today.getMonthValue() && day == today.getDayOfMonth()) {
                calendar.append(String.format("[%2d] ", day)); // Highlight today's date
            } else {
                calendar.append(String.format("%4d ", day));
            }
            if ((day + dayOfWeek) % 7 == 0 || day == daysInMonth) {
                calendar.append("\n");
            }
        }

        return calendar.toString();
    }

    // GUI Implementation
    public static void createAndShowGUI() {
        JFrame frame = new JFrame("✨ Cool Calendar Generator ✨");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(800, 600);
        frame.setLayout(new BorderLayout());

        // Main Panel with Gradient Background
        JPanel mainPanel = new JPanel() {
            @Override
            protected void paintComponent(Graphics g) {
                super.paintComponent(g);
                Graphics2D g2d = (Graphics2D) g;
                Color color1 = new Color(135, 206, 250); // Light Sky Blue
                Color color2 = new Color(30, 144, 255);  // Dodger Blue
                GradientPaint gradient = new GradientPaint(0, 0, color1, getWidth(), getHeight(), color2);
                g2d.setPaint(gradient);
                g2d.fillRect(0, 0, getWidth(), getHeight());
            }
        };
        mainPanel.setLayout(new BorderLayout());
        mainPanel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

        // Calendar Display Area
        JTextArea calendarArea = new JTextArea();
        calendarArea.setEditable(false);
        calendarArea.setFont(new Font("Consolas", Font.PLAIN, 16));
        calendarArea.setBackground(new Color(255, 255, 255, 150)); // Semi-transparent white
        calendarArea.setForeground(new Color(50, 50, 50));
        JScrollPane scrollPane = new JScrollPane(calendarArea);
        scrollPane.setBorder(BorderFactory.createLineBorder(new Color(200, 200, 200, 100), 1, true));
        scrollPane.setOpaque(false);
        scrollPane.getViewport().setOpaque(false);

        // Input Panel
        JPanel inputPanel = new JPanel();
        inputPanel.setLayout(new GridLayout(3, 2, 10, 10));
        inputPanel.setOpaque(false);

        JLabel monthLabel = new JLabel("Month (1-12):");
        monthLabel.setFont(new Font("Arial", Font.BOLD, 16));
        monthLabel.setForeground(Color.WHITE);
        JTextField monthField = new JTextField();
        monthField.setFont(new Font("Arial", Font.PLAIN, 16));
        monthField.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createLineBorder(new Color(200, 200, 200), 1),
                BorderFactory.createEmptyBorder(5, 5, 5, 5)
        ));

        JLabel yearLabel = new JLabel("Year:");
        yearLabel.setFont(new Font("Arial", Font.BOLD, 16));
        yearLabel.setForeground(Color.WHITE);
        JTextField yearField = new JTextField();
        yearField.setFont(new Font("Arial", Font.PLAIN, 16));
        yearField.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createLineBorder(new Color(200, 200, 200), 1),
                BorderFactory.createEmptyBorder(5, 5, 5, 5)
        ));

        JButton viewMonthButton = createStyledButton("View Month", new Color(0, 123, 255));
        JButton viewYearButton = createStyledButton("View Year", new Color(40, 167, 69));

        inputPanel.add(monthLabel);
        inputPanel.add(monthField);
        inputPanel.add(yearLabel);
        inputPanel.add(yearField);
        inputPanel.add(viewMonthButton);
        inputPanel.add(viewYearButton);

        // Add Components to Main Panel
        mainPanel.add(scrollPane, BorderLayout.CENTER);
        mainPanel.add(inputPanel, BorderLayout.SOUTH);

        // Button Actions
        viewMonthButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                try {
                    int month = Integer.parseInt(monthField.getText());
                    int year = Integer.parseInt(yearField.getText());

                    if (month < 1 || month > 12) {
                        calendarArea.setText("Invalid month! Please enter a month between 1 and 12.");
                    } else {
                        calendarArea.setText(getMonthCalendar(month, year));
                    }
                } catch (NumberFormatException ex) {
                    calendarArea.setText("Please enter valid numeric values for month and year.");
                }
            }
        });

        viewYearButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                try {
                    int year = Integer.parseInt(yearField.getText());
                    StringBuilder yearCalendar = new StringBuilder();

                    for (int month = 1; month <= 12; month++) {
                        yearCalendar.append(getMonthCalendar(month, year)).append("\n");
                    }

                    calendarArea.setText(yearCalendar.toString());
                } catch (NumberFormatException ex) {
                    calendarArea.setText("Please enter a valid numeric value for year.");
                }
            }
        });

        // Add Main Panel to Frame
        frame.add(mainPanel);
        frame.setVisible(true);
    }

    // Helper method to create styled buttons
    private static JButton createStyledButton(String text, Color color) {
        JButton button = new JButton(text);
        button.setFont(new Font("Arial", Font.BOLD, 16));
        button.setBackground(color);
        button.setForeground(Color.WHITE);
        button.setFocusPainted(false);
        button.setBorder(BorderFactory.createEmptyBorder(10, 20, 10, 20));
        button.setCursor(new Cursor(Cursor.HAND_CURSOR));

        // Hover Effect
        button.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                button.setBackground(color.darker());
            }

            public void mouseExited(java.awt.event.MouseEvent evt) {
                button.setBackground(color);
            }
        });

        return button;
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(() -> createAndShowGUI());
    }
}